#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "qmessagebox.h"
#include "addappointment.h"
#include "qtimer.h"
#include "qdatetime.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    QFont font("Serif", 8, QFont::Bold);
    for(int i=0; i<7; i++){
        this->ui->calendarView->horizontalHeaderItem(i)->setFont(font);
        this->ui->calendarView->horizontalHeaderItem(i)->setSizeHint(QSize(150, 50));
    }
    for(int j=0; j<6; j++){
        this->ui->calendarView->verticalHeader()->resizeSection(j, 60);
    }

    this->on_btnCurrentWeek_clicked();
    this->updateCalendar();
    this->updateDaily();
    this->updateWeekly();
    this->updateNotification();

    QTimer * timer = new QTimer;
    connect(timer, SIGNAL(timeout()), this, SLOT(onTimerUpdate()));
    timer->start(1000);
    this->onTimerUpdate();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_calendarView_cellClicked(int row, int column)
{

}

void MainWindow::on_calendarView_customContextMenuRequested(QPoint pos)
{
    QTableWidgetItem * item = this->ui->calendarView->itemAt(pos);
    if(!item){
        this->ui->menuMain->exec(this->ui->calendarView->viewport()->mapToGlobal(pos));
    }else{
        this->ui->menuEdit->exec(this->ui->calendarView->viewport()->mapToGlobal(pos));
    }
}

void MainWindow::on_actionMake_triggered()
{
    QList<QTableWidgetSelectionRange> list = this->ui->calendarView->selectedRanges();
    if(list.size() == 0)
        return;
    QTableWidgetSelectionRange r = list[0];
    foreach(QTableWidgetSelectionRange rr, list){
        if(rr.topRow() < r.topRow())
            r = QTableWidgetSelectionRange(rr.topRow(), r.leftColumn(), r.bottomRow(), r.rightColumn());
        if(rr.bottomRow() > r.bottomRow())
            r = QTableWidgetSelectionRange(r.topRow(), r.leftColumn(), rr.bottomRow(), r.rightColumn());
    }

    if(r.leftColumn() != r.rightColumn()){
        QMessageBox::warning(0, "Invalid", "Multiple columns are not allowed.");
        return;
    }
    QString date = this->dayOfWeek.addDays(r.leftColumn()).toString("yyyy-MM-dd");

    AddAppointment add;
    add.setModal(true);
    add.setDate(QString("%2 %1").arg(this->makeTime(r.topRow(), r.bottomRow())).arg(date));
    add.show();
    if(add.exec()){
        if(add.getName().isEmpty()){
            QMessageBox::warning(0, "Invalid", "You must input a valid patient name.");
            return;
        }
        this->appointmentController.make(add.getName(), add.getPhone(), add.getPurpose(), date, r.topRow(), r.bottomRow());
        this->updateCalendar();
    }

}

void MainWindow::onTimerUpdate()
{
    QString time = QString("Current Time: ") + QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
    this->ui->lbCurrentTime->setText(time);
}

void MainWindow::updateCalendar()
{

    char weekName[][15] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};
    for(int i=0; i<7; i++){
        this->ui->calendarView->horizontalHeaderItem(i)->setText(QString(weekName[i]) + tr("\n")
                                                                 + this->dayOfWeek.addDays(i).toString("yyyy-MM-dd") );
    }
    this->ui->calendarView->clearContents();
    this->ui->calendarView->clearSpans();
    for(int j=0; j<7; j++){
        QString date = this->dayOfWeek.addDays(j).toString("yyyy-MM-dd");
        QString curDate = QDate::currentDate().toString("yyyy-MM-dd");
        QList<AppointmentModel*> list = this->appointmentController.find(date);
        foreach(AppointmentModel* a, list){
            QString text = a->name + "\n[" + a->purpose + "]\n" + this->makeTime(a->beginTime, a->endTime);
            QTableWidgetItem* item = new QTableWidgetItem(text);
            if(date > curDate){
                item->setBackgroundColor(QColor(0, 230, 0));
            }else if(date == curDate){
                item->setBackgroundColor(QColor(230, 230, 0));
            }else{
                item->setBackgroundColor(QColor(200, 200, 200));
            }
            this->ui->calendarView->setItem(a->beginTime, j, item);
            this->ui->calendarView->setSpan(a->beginTime, j, a->endTime - a->beginTime + 1, 1);
        }
    }
}

void MainWindow::on_btnCurrentWeek_clicked()
{
    QDate date = QDate::currentDate();
    int weekday = date.dayOfWeek();
    /* Minus 5 on purpose!! */
    this->dayOfWeek = date.addDays(-(weekday - 1 - 5));
    this->updateCalendar();
}

void MainWindow::on_btnPrevWeek_clicked()
{
    this->dayOfWeek = this->dayOfWeek.addDays(-7);
    this->updateCalendar();
}

void MainWindow::on_btnNextWeek_clicked()
{
    this->dayOfWeek = this->dayOfWeek.addDays(7);
    this->updateCalendar();
}

void MainWindow::on_actionEdit_triggered()
{
    QList<QTableWidgetItem*> list = this->ui->calendarView->selectedItems();
    if(list.size() == 0 || list.size() > 1)
        return;
    QTableWidgetItem* item = list[0];
    QString date = this->dayOfWeek.addDays(item->column()).toString("yyyy-MM-dd");
    AppointmentModel* a = this->appointmentController.find(date, item->row());
    if(!a)
        return;

    AddAppointment add;
    add.setModal(true);
    add.setDate(QString("%2 %1").arg(this->makeTime(a->beginTime, a->endTime)).arg(date));
    add.setName(a->name);
    add.setPhone(a->phone);
    add.setPurpose(a->purpose);
    add.show();
    if(add.exec()){
        if(add.getName().isEmpty()){
            QMessageBox::warning(0, "Invalid", "You must input a valid patient name.");
            return;
        }
        this->appointmentController.cancel(a->id);
        this->appointmentController.make(add.getName(), add.getPhone(), add.getPurpose(), date, a->beginTime, a->endTime);
        this->updateCalendar();
    }
}

void MainWindow::on_actionCancel_triggered()
{
    QList<QTableWidgetItem*> list = this->ui->calendarView->selectedItems();
    if(list.size() == 0 || list.size() > 1)
        return;
    QTableWidgetItem* item = list[0];
    QString date = this->dayOfWeek.addDays(item->column()).toString("yyyy-MM-dd");
    AppointmentModel* a = this->appointmentController.find(date, item->row());
    if(!a)
        return;

    QMessageBox msgBox;
    msgBox.addButton(QMessageBox::Yes);
    msgBox.addButton(QMessageBox::No);
    msgBox.setWindowTitle("Cancel Appointment");
    msgBox.setText(QString("Are you sure to cancel %1's appointment?").arg(a->name));
    if(msgBox.exec() == QMessageBox::No)
            return;

    this->appointmentController.cancel(a->id);
    this->updateCalendar();
}

QString MainWindow::makeTime(int begin, int end)
{
    const char startTime[][10] = {"8:00","9:00", "10:00", "14:00", "15:00", "16:00"};
    const char endTime[][10] = {"9:00","10:00", "11:00", "15:00", "16:00", "17:00"};
    return QString("%1~%2").arg(startTime[begin]).arg(endTime[end]);
}

void MainWindow::on_tabWidget_currentChanged(QWidget* tab)
{
}

void MainWindow::on_tabWidget_currentChanged(int index)
{
    switch(index){
    case 0:
        this->updateCalendar();
        break;
    case 1:
        this->updateDaily();
        break;
    case 2:
        this->updateWeekly();
        break;
    case 3:
        this->updateNotification();
        break;
    case 4:
        this->ui->deQueryDate->setDate(QDate::currentDate());
        break;
    }
}

void MainWindow::on_MainWindow_destroyed()
{
}

void MainWindow::closeEvent(QCloseEvent *)
{
    QMessageBox msgBox;
    msgBox.addButton(QMessageBox::Yes);
    msgBox.addButton(QMessageBox::No);
    msgBox.setText(QString("Do you want to save the changes of this session?"));
    if(msgBox.exec() == QMessageBox::No)
            return;
    this->appointmentController.save();
}

void MainWindow::updateDaily()
{
    this->ui->dailyView->clearContents();
    this->ui->dailyView->horizontalHeader()->resizeSection(5, 300);
    int row = 0;
    QString today = QDate::currentDate().toString("yyyy-MM-dd");
    for(int i=0; i<6; i++){
        AppointmentModel * a = this->appointmentController.find(today, i);
        if(a){
            this->ui->dailyView->setRowCount(row+1);
            this->ui->dailyView->setItem(row, 0, new QTableWidgetItem(QString("%1) ").arg(a->beginTime)
                                                                      + this->makeTime(a->beginTime, a->endTime)));
            this->ui->dailyView->setItem(row, 1, new QTableWidgetItem(a->name));
            this->ui->dailyView->setItem(row, 2, new QTableWidgetItem(a->purpose));
            this->ui->dailyView->setItem(row, 3, new QTableWidgetItem(a->phone));
            this->ui->dailyView->setItem(row, 4, new QTableWidgetItem(a->state));
            this->ui->dailyView->setItem(row, 5, new QTableWidgetItem(a->comment));
            row += 1;
        }
    }
    this->ui->grpMarkApp->setEnabled(false);
}

void MainWindow::updateWeekly()
{
    this->ui->weeklyView->clearContents();
    this->ui->weeklyView->horizontalHeader()->resizeSection(0, 160);
    this->ui->weeklyView->horizontalHeader()->resizeSection(5, 200);
    int row = 0;
    for(int k=0; k<7; k++){
        QString day = this->dayOfWeek.addDays(k).toString("yyyy-MM-dd");
        for(int i=0; i<6; i++){
            AppointmentModel * a = this->appointmentController.find(day, i);
            if(a){
                this->ui->weeklyView->setRowCount(row+1);
                this->ui->weeklyView->setItem(row, 0, new QTableWidgetItem(QString("%1 ").arg(day)
                                                                          + this->makeTime(a->beginTime, a->endTime)));
                this->ui->weeklyView->setItem(row, 1, new QTableWidgetItem(a->name));
                this->ui->weeklyView->setItem(row, 2, new QTableWidgetItem(a->purpose));
                this->ui->weeklyView->setItem(row, 3, new QTableWidgetItem(a->phone));
                this->ui->weeklyView->setItem(row, 4, new QTableWidgetItem(a->state));
                this->ui->weeklyView->setItem(row, 5, new QTableWidgetItem(a->comment));
                row += 1;
            }
        }
    }
}

void MainWindow::updateNotification()
{
    this->ui->notificationView->clearContents();
    this->ui->notificationView->horizontalHeader()->resizeSection(0, 200);
    this->ui->notificationView->horizontalHeader()->resizeSection(3, 200);
    int row = 0;
    for(int k=1; k<3; k++){
        QString day = QDate::currentDate().addDays(k).toString("yyyy-MM-dd");
        for(int i=0; i<6; i++){
            AppointmentModel * a = this->appointmentController.find(day, i);
            if(a){
                this->ui->notificationView->setRowCount(row+1);
                this->ui->notificationView->setItem(row, 0, new QTableWidgetItem(QString("%1 ").arg(day)
                                                                          + this->makeTime(a->beginTime, a->endTime)));
                this->ui->notificationView->setItem(row, 1, new QTableWidgetItem(a->name));
                this->ui->notificationView->setItem(row, 2, new QTableWidgetItem(a->purpose));
                this->ui->notificationView->setItem(row, 3, new QTableWidgetItem(a->phone));
                row += 1;
            }
        }
    }
}

void MainWindow::on_dailyView_clicked(QModelIndex index)
{
    QString today = QDate::currentDate().toString("yyyy-MM-dd");
    QTableWidgetItem* item = this->ui->dailyView->item(index.row(), 0);
    if(item){
        int beginTime = QString(item->text()[0]).toInt();
        AppointmentModel* a = this->appointmentController.find(today, beginTime);
        if(a){
            this->ui->txtPatientId->setText(QString("%1").arg(a->id));
            this->ui->txtPatientName->setText(a->name);
            this->ui->txtPurpose->setText(a->purpose);
            this->ui->txtComment->setText(a->comment);
            this->ui->cbState->setEditText(a->state);
            this->ui->grpMarkApp->setEnabled(true);
        }
    }
}

void MainWindow::on_btnSaveMark_clicked()
{
    int patientId = this->ui->txtPatientId->text().toInt();
    AppointmentModel* a = this->appointmentController.find(patientId);
    if(a){
        a->comment = this->ui->txtComment->toPlainText();
        a->state = this->ui->cbState->currentText();
        this->updateDaily();
    }
}

void MainWindow::on_btnSearchName_clicked()
{
    this->ui->queryView->clearContents();
    this->ui->queryView->horizontalHeader()->resizeSection(0, 160);
    this->ui->queryView->horizontalHeader()->resizeSection(5, 200);
    int row = 0;
    QString queryName = this->ui->txtQueryName->text().toLower();
    foreach(AppointmentModel* a, this->appointmentController.find()){
        if(a->name.toLower().indexOf(queryName) >= 0){
            this->ui->queryView->setRowCount(row+1);
            this->ui->queryView->setItem(row, 0, new QTableWidgetItem(QString("%1 ").arg(a->date)
                                                                      + this->makeTime(a->beginTime, a->endTime)));
            this->ui->queryView->setItem(row, 1, new QTableWidgetItem(a->name));
            this->ui->queryView->setItem(row, 2, new QTableWidgetItem(a->purpose));
            this->ui->queryView->setItem(row, 3, new QTableWidgetItem(a->phone));
            this->ui->queryView->setItem(row, 4, new QTableWidgetItem(a->state));
            this->ui->queryView->setItem(row, 5, new QTableWidgetItem(a->comment));
            row += 1;
        }
    }
    this->ui->queryView->sortByColumn(0);
}

void MainWindow::on_btnSearchDate_clicked()
{
    this->ui->queryView->clearContents();
    this->ui->queryView->horizontalHeader()->resizeSection(0, 160);
    this->ui->queryView->horizontalHeader()->resizeSection(5, 200);
    int row = 0;
    QString queryDate = this->ui->deQueryDate->date().toString("yyyy-MM-dd");
    foreach(AppointmentModel* a, this->appointmentController.find()){
        if(a->date == queryDate){
            this->ui->queryView->setRowCount(row+1);
            this->ui->queryView->setItem(row, 0, new QTableWidgetItem(QString("%1 ").arg(a->date)
                                                                      + this->makeTime(a->beginTime, a->endTime)));
            this->ui->queryView->setItem(row, 1, new QTableWidgetItem(a->name));
            this->ui->queryView->setItem(row, 2, new QTableWidgetItem(a->purpose));
            this->ui->queryView->setItem(row, 3, new QTableWidgetItem(a->phone));
            this->ui->queryView->setItem(row, 4, new QTableWidgetItem(a->state));
            this->ui->queryView->setItem(row, 5, new QTableWidgetItem(a->comment));
            row += 1;
        }
    }
}
